@@ -66,6 +66,7 @@ rect getcliprect(void);
 void setcliprect(rect r);
 PROTECTED void updatestatus(const char *text);
 PROTECTED font new_font_object(HFONT hf);
+UINT default_font_charset();
 
 #ifdef __cplusplus
 extern "C" {
@@ -231,6 +232,10 @@ struct objinfo
 #if USE_NATIVE_CONTROLS
     HBRUSH  bgbrush; /* background brush */
 #endif
+    int caretwidth;  /* Caret (blinking cursor) is set here. caretwidth=0 for none, 
+                        negative for not focussed */
+    int caretheight;
+    int caretshowing;
 };
 
 struct callinfo
