@@ -1,7 +1,7 @@
 #  File src/library/grDevices/R/xyz.coords.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,60 +14,69 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 ## Both xy.coords() and xyz.coords()  --- should be kept in sync!
 
 xy.coords <-
-    function(x, y=NULL, xlab=NULL, ylab=NULL, log=NULL, recycle = FALSE)
+    function(x, y = NULL, xlab = NULL, ylab = NULL, log = NULL, recycle = FALSE,
+             setLab = TRUE)
 {
     if(is.null(y)) {
-	ylab <- xlab
+	if(is.null(ylab)) ylab <- xlab
 	if(is.language(x)) {
 	    if (inherits(x, "formula") && length(x) == 3) {
-		ylab <- deparse(x[[2L]])
-		xlab <- deparse(x[[3L]])
-		y <- eval(x[[2L]], environment(x), parent.frame())
-		x <- eval(x[[3L]], environment(x), parent.frame())
+                if(setLab) {
+                    ylab <- deparse(x[[2L]])
+                    xlab <- deparse(x[[3L]])
+                }
+		y <- eval(x[[2L]], environment(x))
+		x <- eval(x[[3L]], environment(x))
 	    }
 	    else stop("invalid first argument")
 	}
 	else if(inherits(x, "ts")) {
 	    y <- if(is.matrix(x)) x[,1] else x
 	    x <- stats::time(x)
-	    xlab <- "Time"
+            if(setLab) xlab <- "Time"
 	}
 	else if(is.complex(x)) {
 	    y <- Im(x)
 	    x <- Re(x)
-	    xlab <- paste0("Re(", ylab, ")")
-	    ylab <- paste0("Im(", ylab, ")")
+            if(setLab) {
+                xlab <- paste0("Re(", ylab, ")")
+                ylab <- paste0("Im(", ylab, ")")
+            }
 	}
 	else if(is.matrix(x) || is.data.frame(x)) {
 	    x <- data.matrix(x)
 	    if(ncol(x) == 1) {
-		xlab <- "Index"
+		if(setLab) xlab <- "Index"
 		y <- x[,1]
 		x <- seq_along(y)
 	    }
 	    else {
 		colnames <- dimnames(x)[[2L]]
-		if(is.null(colnames)) {
-		    xlab <- paste0(ylab, "[,1]")
-		    ylab <- paste0(ylab, "[,2]")
-		}
-		else {
-		    xlab <- colnames[1L]
-		    ylab <- colnames[2L]
-		}
+                if(setLab) {
+                    if(is.null(colnames)) {
+                        xlab <- paste0(ylab, "[,1]")
+                        ylab <- paste0(ylab, "[,2]")
+                    }
+                    else {
+                        xlab <- colnames[1L]
+                        ylab <- colnames[2L]
+                    }
+                }
 		y <- x[,2]
 		x <- x[,1]
 	    }
 	}
 	else if(is.list(x)) {
             if (all(c("x", "y") %in% names(x))) {
-                xlab <- paste0(ylab, "$x")
-                ylab <- paste0(ylab, "$y")
+                if(setLab) {
+                    xlab <- paste0(ylab, "$x")
+                    ylab <- paste0(ylab, "$y")
+                }
                 y <- x[["y"]]
                 x <- x[["x"]]
             } else
@@ -75,7 +84,7 @@ xy.coords <-
 	}
 	else {
 	    if(is.factor(x)) x <- as.numeric(x)
-	    xlab <- "Index"
+	    if(setLab) xlab <- "Index"
 	    y <- x
 	    x <- seq_along(x)
 	}
@@ -99,34 +108,36 @@ xy.coords <-
 	if("x" %in% log && any(ii <- x <= 0 & !is.na(x))) {
 	    n <- as.integer(sum(ii))
 	    warning(sprintf(ngettext(n,
-                            "%d x value <= 0 omitted from logarithmic plot",
-                            "%d x values <= 0 omitted from logarithmic plot"),
+                                     "%d x value <= 0 omitted from logarithmic plot",
+                                     "%d x values <= 0 omitted from logarithmic plot"),
                             n), domain = NA)
 	    x[ii] <- NA
 	}
 	if("y" %in% log && any(ii <- y <= 0 & !is.na(y))) {
 	    n <- as.integer(sum(ii))
 	    warning(sprintf(ngettext(n,
-                            "%d y value <= 0 omitted from logarithmic plot",
-                            "%d y values <= 0 omitted from logarithmic plot"),
+                                     "%d y value <= 0 omitted from logarithmic plot",
+                                     "%d y values <= 0 omitted from logarithmic plot"),
                             n), domain = NA)
 	    y[ii] <- NA
 	}
     }
-    return(list(x=as.double(x), y=as.double(y), xlab=xlab, ylab=ylab))
+    list(x=as.double(x), y=as.double(y), xlab=xlab, ylab=ylab)
 }
 
 xyz.coords <- function(x, y=NULL, z=NULL, xlab=NULL, ylab=NULL, zlab=NULL,
-		       log = NULL, recycle = FALSE)
+		       log = NULL, recycle = FALSE, setLab = TRUE)
 {
     ## Only x
     if(is.null(y)) {
 	if (is.language(x)) {
 	    if (inherits(x, "formula") && length(x) == 3
 		&& length(rhs <- x[[3L]]) == 3) {
-		zlab <- deparse(x[[2L]])
-		ylab <- deparse(rhs[[3L]])
-		xlab <- deparse(rhs[[2L]])
+                if(setLab) {
+                    zlab <- deparse(x[[2L]])
+                    ylab <- deparse(rhs[[3L]])
+                    xlab <- deparse(rhs[[2L]])
+                }
 		pf <- parent.frame()
 		z <- eval(x[[2L]],   environment(x), pf)
 		y <- eval(rhs[[3L]], environment(x), pf)
@@ -138,23 +149,25 @@ xyz.coords <- function(x, y=NULL, z=NULL, xlab=NULL, ylab=NULL, zlab=NULL,
 	    x <- data.matrix(x)
 	    if(ncol(x) < 2) stop("at least 2 columns needed")
 	    if(ncol(x) == 2) {
-		xlab <- "Index"
+		if(setLab) xlab <- "Index"
 		y <- x[,1]
 		z <- x[,2]
 		x <- seq_along(y)
 	    }
 	    else { ## >= 3 columns
 		colnames <- dimnames(x)[[2L]]
-		if(is.null(colnames)) {
-		    zlab <- paste0(xlab,"[,3]")
-		    ylab <- paste0(xlab,"[,2]")
-		    xlab <- paste0(xlab,"[,1]")
-		}
-		else {
-		    xlab <- colnames[1L]
-		    ylab <- colnames[2L]
-		    zlab <- colnames[3L]
-		}
+                if(setLab) {
+                    if(is.null(colnames)) {
+                        zlab <- paste0(xlab,"[,3]")
+                        ylab <- paste0(xlab,"[,2]")
+                        xlab <- paste0(xlab,"[,1]")
+                    }
+                    else {
+                        xlab <- colnames[1L]
+                        ylab <- colnames[2L]
+                        zlab <- colnames[3L]
+                    }
+                }
 		y <- x[,2]
 		z <- x[,3]
 		x <- x[,1]
@@ -162,9 +175,11 @@ xyz.coords <- function(x, y=NULL, z=NULL, xlab=NULL, ylab=NULL, zlab=NULL,
 	}
 	else if(is.list(x)) {
             if (all(c("x", "y", "z") %in% names(x))) {
-                zlab <- paste0(xlab,"$z")
-                ylab <- paste0(xlab,"$y")
-                xlab <- paste0(xlab,"$x")
+                if(setLab) {
+                    zlab <- paste0(xlab,"$z")
+                    ylab <- paste0(xlab,"$y")
+                    xlab <- paste0(xlab,"$x")
+                }
                 y <- x[["y"]]
                 z <- x[["z"]]
                 x <- x[["x"]]
@@ -179,22 +194,26 @@ xyz.coords <- function(x, y=NULL, z=NULL, xlab=NULL, ylab=NULL, zlab=NULL,
 	    z <- y
 	    y <- Im(x)
 	    x <- Re(x)
-	    zlab <- ylab
-	    ylab <- paste0("Im(", xlab, ")")
-	    xlab <- paste0("Re(", xlab, ")")
+            if(setLab) {
+                zlab <- ylab
+                ylab <- paste0("Im(", xlab, ")")
+                xlab <- paste0("Re(", xlab, ")")
+            }
 	}
 	else if(is.complex(y)) {
 	    z <- x
 	    x <- Re(y)
 	    y <- Im(y)
-	    zlab <- xlab
-	    xlab <- paste0("Re(", ylab, ")")
-	    ylab <- paste0("Im(", ylab, ")")
+            if(setLab) {
+                zlab <- xlab
+                xlab <- paste0("Re(", ylab, ")")
+                ylab <- paste0("Im(", ylab, ")")
+            }
 	}
 	else {
 	    if(is.factor(x)) x <- as.numeric(x)
 	    if(is.factor(y)) y <- as.numeric(y)
-	    xlab <- "Index"
+            if(setLab) xlab <- "Index"
 	    z <- y
 	    y <- x
 	    x <- seq_along(x)
