@@ -626,6 +626,8 @@ static cairo_pattern_t *CairoCreateMask(SEXP mask, pX11Desc xd)
     SEXP R_fcall;
     /* Start new group - drawing is redirected to this group */
     cairo_push_group(cc);
+    /* Start with "over" operator */
+    cairo_set_operator(cc, CAIRO_OPERATOR_OVER);
     /* Play the mask function to build the mask */
     R_fcall = PROTECT(lang1(mask));
     eval(R_fcall, R_GlobalEnv);
@@ -704,6 +706,7 @@ static void CairoInitGroups(pX11Desc xd)
     for (i = 0; i < xd->numGroups; i++) {
         xd->groups[i] = NULL;
     }
+    xd->nullGroup = cairo_pattern_create_rgb(0, 0, 0);
 }
 
 static int CairoGrowGroups(pX11Desc xd)
@@ -727,7 +730,8 @@ static void CairoCleanGroups(pX11Desc xd)
 {
     int i;
     for (i = 0; i < xd->numGroups; i++) {
-        if (xd->groups[i] != NULL) {
+        if (xd->groups[i] != NULL &&
+            xd->groups[i] != xd->nullGroup) {
             cairo_pattern_destroy(xd->groups[i]);
             xd->groups[i] = NULL;
         }
@@ -738,12 +742,14 @@ static void CairoDestroyGroups(pX11Desc xd)
 {
     int i;
     for (i = 0; i < xd->numGroups; i++) {
-        if (xd->groups[i] != NULL) {
+        if (xd->groups[i] != NULL &&
+            xd->groups[i] != xd->nullGroup) {
             cairo_pattern_destroy(xd->groups[i]);
             xd->groups[i] = NULL;
         }
     }    
     free(xd->groups);
+    cairo_pattern_destroy(xd->nullGroup);
 }
 
 static int CairoNewGroupIndex(pX11Desc xd)
@@ -751,6 +757,9 @@ static int CairoNewGroupIndex(pX11Desc xd)
     int i;
     for (i = 0; i < xd->numGroups; i++) {
         if (xd->groups[i] == NULL) {
+            /* Place temporary hold on this slot in case of 
+             * group within group */
+            xd->groups[i] = xd->nullGroup;
             return i;
         } else {
             if (i == (xd->numGroups - 1) &&
@@ -806,6 +815,8 @@ static cairo_pattern_t *CairoCreateGroup(SEXP src, int op, SEXP dst,
 
     /* Start new group - drawing is redirected to this group */
     cairo_push_group(cc);
+    /* Start with "over" operator */
+    cairo_set_operator(cc, CAIRO_OPERATOR_OVER);
 
     if (dst != R_NilValue) {
         /* Play the destination function to draw the destination */
@@ -1293,7 +1304,8 @@ static void Cairo_Raster(unsigned int *raster, int w, int h,
      * the image used for the Cairo surface is going to be a
      * different size
      */
-    if (interpolate && CAIRO_VERSION_MAJOR < 2 && CAIRO_VERSION_MINOR < 6) {
+    // clang 15 prefers & here for constant expressions.
+    if (interpolate & (CAIRO_VERSION_MAJOR < 2) & (CAIRO_VERSION_MINOR < 6)) {
         imageWidth = (int) (width + .5);
         imageHeight = abs((int) (height + .5));
     } else {    
@@ -1310,7 +1322,8 @@ static void Cairo_Raster(unsigned int *raster, int w, int h,
     cairo_translate(xd->cc, 0, -imageHeight/2.0);
 
     if (interpolate) {
-        if (CAIRO_VERSION_MAJOR < 2 && CAIRO_VERSION_MINOR < 6) {
+	// clang 15 prefers & here for constant expressions.
+        if ((CAIRO_VERSION_MAJOR < 2) & (CAIRO_VERSION_MINOR < 6)) {
             /* CAIRO_EXTEND_PAD not supported for image sources 
              * so use graphics engine for interpolation 
              */
