@@ -1,7 +1,7 @@
 #  File src/library/methods/R/as.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2018 The R Core Team
+#  Copyright (C) 1995-2019 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -200,7 +200,8 @@ setAs <-
 {
     ## where there is an "is" relation, modify it
     fromDef <- getClassDef(from, where)
-    extds <- possibleExtends(from, to, fromDef)
+    toDef <- getClassDef(to, where)
+    extds <- possibleExtends(from, to, fromDef, toDef)
     if(is(extds, "SClassExtension")) {
         test <- extds@test
         if(is.null(replace))
@@ -215,7 +216,6 @@ setAs <-
                  domain = NA)
         ## usually to will be a class union, where setAs() is not
         ## allowed by the definition of a union
-        toDef <- getClassDef(to, where=where)
         if(is.null(toDef))
             stop(gettextf("class %s is not defined in this environment",
                           dQuote(to)),
@@ -407,9 +407,9 @@ setAs <-
                              dQuote(to)),
                     domain = NA)
         if(prevCoerce)
-            setMethod(cdef, sig, NULL, where = baseenv())
+            setMethod(cdef, sig, NULL, where = where)
         if(prevRepl)
-            setMethod(rdef, sig, NULL, where = baseenv())
+            setMethod(rdef, sig, NULL, where = where)
         TRUE
     }
     else
@@ -419,7 +419,7 @@ setAs <-
 
 canCoerce <- function(object, Class) {
     is(object, Class) ||
-    !is.null(selectMethod("coerce", c(class(object), Class),
+    !is.null(selectMethod("coerce", c(.class1(object), Class),
 			  optional = TRUE,
 			  useInherited = c(from=TRUE, to=FALSE)))
 }
