@@ -1,7 +1,7 @@
 #  File src/library/tools/R/parseRd.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 
 parse_Rd <- function(file, srcfile = NULL, encoding = "unknown",
                      verbose = FALSE, fragment = FALSE,
-                     warningCalls = TRUE, 
+                     warningCalls = TRUE,
                      macros = file.path(R.home("share"), "Rd", "macros", "system.Rd"),
 		     permissive = FALSE)
 {
@@ -34,7 +34,7 @@ parse_Rd <- function(file, srcfile = NULL, encoding = "unknown",
         }
     } else file0 <- "<connection>"
     lines <- readLines(file, warn = FALSE)
-    if(is.character(macros)) 
+    if(is.character(macros))
     	macros <- initialRdMacros(macros = macros)
     ## remove old-style marking for data, keep line nos
     lines[lines == "\\non_function{}"] <- ""
@@ -42,8 +42,8 @@ parse_Rd <- function(file, srcfile = NULL, encoding = "unknown",
     ## do this in two steps to minimize warnings in MBCS locales
     ## Note this is required to be on a line by itself,
     ## but some people have preceding whitespace
-    enc <- grep("\\encoding{", lines, fixed = TRUE, useBytes=TRUE)
-    enc <- grep("^[[:space:]]*\\\\encoding\\{([^}]*)\\}.*", lines[enc], value=TRUE)
+    enc <- grep("\\encoding{", lines, fixed = TRUE, useBytes=TRUE, value=TRUE)
+    enc <- grep("^[[:space:]]*\\\\encoding\\{([^}]*)\\}.*", enc, value = TRUE)
     if(length(enc)) {
         if(length(enc) > 1L)
             warning(file0, ": multiple \\encoding lines, using the first",
@@ -54,7 +54,7 @@ parse_Rd <- function(file, srcfile = NULL, encoding = "unknown",
         if(verbose) message("found encoding ", enc, domain = NA)
         encoding <- if(enc %in% c("UTF-8", "utf-8", "utf8")) "UTF-8" else enc
     }
-    if (encoding == "unknown") encoding <- ""
+    if (length(encoding) != 1L || encoding == "unknown") encoding <- ""
 
     ## the internal function must get some sort of srcfile
     if (!inherits(srcfile, "srcfile"))
@@ -64,50 +64,61 @@ parse_Rd <- function(file, srcfile = NULL, encoding = "unknown",
     srcfile$Enc <- "UTF-8"
 
     if (encoding == "ASCII") {
-        if (any(is.na(iconv(lines, "", "ASCII"))))
+        if (anyNA(iconv(lines, "", "ASCII")))
             stop(file0, ": non-ASCII input and no declared encoding",
                  domain = NA, call. = warningCalls)
-    } else if (encoding != "UTF-8")
-    	lines <- iconv(lines, encoding, "UTF-8", sub = "byte")
+    } else {
+	if (encoding != "UTF-8")
+    	    lines <- iconv(lines, encoding, "UTF-8", sub = "byte")
+        ## Strip UTF-8 BOM if necessary.
+        bytes <- charToRaw(lines[1L])
+        if(identical(as.integer(bytes[1L : 3L]),
+                     c(0xefL, 0xbbL, 0xbfL)))
+            lines[1L] <- rawToChar(bytes[-(1L : 3L)])
+    }
 
     tcon <- file()
     writeLines(lines, tcon, useBytes = TRUE)
     on.exit(close(tcon))
-    
+
     warndups <- config_val_to_logical(Sys.getenv("_R_WARN_DUPLICATE_RD_MACROS_", "FALSE"))
 
-    if (permissive) 
-	# FIXME:  this should test for a special class of warning rather than testing the
-	#         message, but those are currently not easily generated from C code.
-	result <- withCallingHandlers(.External2(C_parseRd, tcon, srcfile, "UTF-8",
-                            verbose, basename, fragment, warningCalls, macros, warndups),
-		       warning = function(w) 
+    result <- if(permissive)
+                  ## FIXME:  this should test for a special class of warning rather than testing the
+                  ##         message, but those are currently not easily generated from C code.
+                  withCallingHandlers(.External2(C_parseRd, tcon, srcfile, "UTF-8",
+                                                 verbose, basename, fragment,
+                                                 warningCalls, macros, warndups),
+		       warning = function(w)
 			    if (grepl("unknown macro", conditionMessage(w)))
-				invokeRestart("muffleWarning") )
-    else
-	result <- .External2(C_parseRd, tcon, srcfile, "UTF-8",
-                         verbose, basename, fragment, warningCalls, macros, warndups)
+				tryInvokeRestart("muffleWarning"))
+              else
+                  .External2(C_parseRd, tcon, srcfile, "UTF-8",
+                             verbose, basename, fragment, warningCalls,
+                             macros, warndups)
     result <- expandDynamicFlags(result)
     if (permissive)
-	result <- permissify(result)
-    result
+	permissify(result)
+    else
+        result
 }
 
 print.Rd <- function(x, deparse = FALSE, ...)
 {
-    cat(as.character.Rd(x, deparse = deparse), sep = "", collapse = "")
+    cat(as.character.Rd(x, deparse = deparse), sep = "")
     invisible(x)
 }
 
 as.character.Rd <- function(x, deparse = FALSE, ...)
 {
     ZEROARG <- c("\\cr", "\\dots", "\\ldots", "\\R", "\\tab") # Only these cause trouble when {} is added
-    TWOARG <- c("\\section", "\\item", "\\enc", "\\method", "\\S3method",
-                "\\S4method", "\\tabular")
+    TWOARG <- c("\\section", "\\subsection", "\\item", "\\enc",
+                "\\method", "\\S3method", "\\S4method", "\\tabular",
+                "\\if", "\\href")
     USERMACROS <- c("USERMACRO", "\\newcommand", "\\renewcommand")
     EQN <- c("\\deqn", "\\eqn", "\\figure")
     modes <- c(RLIKE = 1L, LATEXLIKE = 2L, VERBATIM = 3L, INOPTION = 4L, COMMENTMODE = 5L, UNKNOWNMODE = 6L)
-    tags  <- c(RCODE = 1L, TEXT = 2L,      VERB = 3L,                  COMMENT = 5L,     UNKNOWN = 6L)
+    tags  <- c(RCODE = 1L, TEXT      = 2L, VERB     = 3L,                COMMENT     = 5L, UNKNOWN     = 6L)
     state <- c(braceDepth = 0L, inRString = 0L)
     needBraces <- FALSE  # if next character is alphabetic, separate by braces.
     inEqn <- 0L
@@ -123,7 +134,7 @@ as.character.Rd <- function(x, deparse = FALSE, ...)
     	        result <- character()
     	    	for (i in seq_along(x))
                     result <- c(result, pr(x[[i]], quoteBraces))
-    	    } else if (length(grep("^#", tag))) {
+    	    } else if (startsWith(tag, "#")) {
     	    	if (deparse) {
     	    	    dep <- deparseRdElement(x[[1L]][[1L]],
                                             c(state, modes["LATEXLIKE"],
@@ -186,12 +197,13 @@ as.character.Rd <- function(x, deparse = FALSE, ...)
 
 deparseRdElement <- function(element, state)
     .Call(C_deparseRd, element, state)
-    
+
 # Convert unknown tags into text displaying the tag with braces if necessary
 # This allows unknown LateX macros to be embedded in the text, and to be just passed
 # through.
 
-permissify <- function(Rd) {  
+permissify <- function(Rd)
+{
     tags <- RdTags(Rd)
     oldclass <- class(Rd)
     oldsrcref <- utils::getSrcref(Rd)
@@ -203,9 +215,10 @@ permissify <- function(Rd) {
    	    Rd[[i]] <- tagged(Rd[[i]], "TEXT", utils::getSrcref(Rd[[i]]))
             while (i < length(tags)) {
 		if (tags[i+1] == "LIST") {
-		    Rd <- c(Rd[seq_len(i)], list(tagged("{", "TEXT", utils::getSrcref(Rd[[i+1]]))), 
-		                        permissify(Rd[[i+1]]), 
-            	                        list(tagged("}", "TEXT", utils::getSrcref(Rd[[i+1]]))), 
+		    Rd <- c(Rd[seq_len(i)],
+                            list(tagged("{", "TEXT", utils::getSrcref(Rd[[i+1]]))),
+                            permissify(Rd[[i+1]]),
+                            list(tagged("}", "TEXT", utils::getSrcref(Rd[[i+1]]))),
 			    Rd[seq_along(Rd)[-seq_len(i+1)]])
 		    tags <- RdTags(Rd)
 		    i <- i+3
