@@ -1,6 +1,6 @@
 /*
  *   R : A Computer Language for Statistical Data Analysis
- *   Copyright (C) 1997-2015   The R Core Team
+ *   Copyright (C) 1997-2021   The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,15 +23,80 @@
  *  Formerly part of ../unix/sys-common.c.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <ctype.h>
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+
+/* RENVIRON_WIN32_STANDALONE is set when compiling for use in Rcmd Windows
+   front-end, which is not linked against the R library. The locale is not
+   initialized in the front-end.
+
+   When RENVIRON_WIN32_STANDALONE is not set, the code below can be used
+   both when R is starting up (to read standard Renviron files) as well
+   as when R is already running (e.g. to read custom Renviron files).
+   In the former case, the locale is not yet initialized and R warnings/errors
+   can not be emitted, yet.
+
+   When the locale is not initialized, the MBCS-aware functions below will 
+   behave as MBCS-unaware, because mbcslocale will not be set. The reading of
+   Renviron when R is starting up (on all platforms) would hence have issues
+   with some characters in non-UTF-8 MBCS locale (when bytes can be confused
+   for ASCII). Error messages may be imperfectly truncated also in UTF-8. 
+*/
+#ifdef RENVIRON_WIN32_STANDALONE
+
+/* #undef HAVE_SETENV */
+# define HAVE_PUTENV 1
+
+# define Renviron_strchr strchr
+# define Renviron_snprintf snprintf
+
+static void Renviron_warning(const char *msg)
+{
+    fprintf(stderr, "%s\n", msg);
+}
+
+static void Renviron_error(const char *msg)
+{
+    fprintf(stderr, "FATAL ERROR:%s\n", msg);
+    exit(2);
+}
+
+#else /* not RENVIRON_WIN32_STANDALONE */
+
+# ifdef HAVE_CONFIG_H
+#  include <config.h>
+# endif
+# include <Defn.h>
+# include <Fileio.h>
+# include <Rinterface.h>
+
+# ifdef Win32
+#  include <trioremap.h> /* to ensure snprintf result is null terminated */
+# endif
+
+# define Renviron_strchr Rf_strchr
+# define Renviron_snprintf Rsnprintf_mbcs
+
+static void Renviron_warning(const char *msg)
+{
+    if (R_Is_Running > 1)
+	warningcall(R_NilValue, msg);
+    else
+	R_ShowMessage(msg);
+}
+
+static void Renviron_error(const char *msg)
+{
+    if (R_Is_Running > 1)
+	errorcall(R_NilValue, msg);
+    else
+	R_Suicide(msg);
+}
+
 #endif
 
-#include <stdlib.h> /* for setenv or putenv */
-#include <Defn.h> /* for PATH_MAX */
-#include <Rinterface.h>
-#include <Fileio.h>
-#include <ctype.h>		/* for isspace */
 
 /* remove leading and trailing space */
 static char *rmspace(char *s)
@@ -59,7 +124,7 @@ static char *subterm(char *s)
     s += 2;
     s = rmspace(s);
     if(!strlen(s)) return "";
-    p = Rf_strchr(s, '-');
+    p = Renviron_strchr(s, '-');
     if(p) {
 	q = p + 1; /* start of value */
 	if(p - s > 1 && *(p-1) == ':') {
@@ -83,8 +148,8 @@ static char *findRbrace(char *s)
     int nl = 0, nr = 0;
 
     while(nr <= nl) {
-	pl = Rf_strchr(p, '{');
-	pr = Rf_strchr(p, '}');
+	pl = Renviron_strchr(p, '{');
+	pr = Renviron_strchr(p, '}');
 	if(!pr) return NULL;
 	if(!pl || pr < pl) {
 	    p = pr+1; nr++;
@@ -95,7 +160,7 @@ static char *findRbrace(char *s)
     return pr;
 }
 
-#define BUF_SIZE 10000
+#define BUF_SIZE 100000
 static char *findterm(char *s)
 {
     char *p, *q, *r2, *ss=s;
@@ -105,7 +170,7 @@ static char *findterm(char *s)
     ans[0] = '\0';
     while(1) {
 	/* Look for ${...}, taking care to look for inner matches */
-	p = Rf_strchr(s, '$');
+	p = Renviron_strchr(s, '$');
 	if(!p || p[1] != '{') break;
 	q = findRbrace(p+2);
 	if(!q) break;
@@ -128,14 +193,15 @@ static void Putenv(char *a, char *b)
 {
     char *buf, *value, *p, *q, quote='\0';
     int inquote = 0;
+    int failed = 0;
 
 #ifdef HAVE_SETENV
     buf = (char *) malloc((strlen(b) + 1) * sizeof(char));
-    if(!buf) R_Suicide("allocation failure in reading Renviron");
+    if(!buf) Renviron_error("allocation failure in reading Renviron");
     value = buf;
 #else
     buf = (char *) malloc((strlen(a) + strlen(b) + 2) * sizeof(char));
-    if(!buf) R_Suicide("allocation failure in reading Renviron");
+    if(!buf) Renviron_error("allocation failure in reading Renviron");
     strcpy(buf, a); strcat(buf, "=");
     value = buf+strlen(buf);
 #endif
@@ -143,7 +209,9 @@ static void Putenv(char *a, char *b)
     /* now process the value */
     for(p = b, q = value; *p; p++) {
 	/* remove quotes around sections, preserve \ inside quotes */
-	if(!inquote && (*p == '"' || *p == '\'')) {
+	if(!inquote && (*p == '"' || *p == '\'') &&
+	   (p == b || *(p-1) != '\\')) {
+
 	    inquote = 1;
 	    quote = *p;
 	    continue;
@@ -163,66 +231,124 @@ static void Putenv(char *a, char *b)
     *q = '\0';
 #ifdef HAVE_SETENV
     if(setenv(a, buf, 1))
-	warningcall(R_NilValue,
-		    _("problem in setting variable '%s' in Renviron"), a);
+	failed = 1;
     free(buf);
 #elif defined(HAVE_PUTENV)
     if(putenv(buf))
-	warningcall(R_NilValue,
-		    _("problem in setting variable '%s' in Renviron"), a);
+	failed = 1;
     /* no free here: storage remains in use */
 #else
     /* pretty pointless, and was not tested prior to 2.3.0 */
     free(buf);
 #endif
+    if (failed) {
+	char buf[1024];
+	Renviron_snprintf(buf, 1024,
+#ifdef RENVIRON_WIN32_STANDALONE
+	                  "Problem in setting variable '%s' in Renviron",
+#else
+	                  _("problem in setting variable '%s' in Renviron"),
+#endif
+	                  a);
+	Renviron_warning(buf);
+    }
 }
 
 
-#define MSG_SIZE 2000
+#define MSG_SIZE 2048
+
+#ifdef RENVIRON_WIN32_STANDALONE
+int process_Renviron(const char *filename)
+#else
 static int process_Renviron(const char *filename)
+#endif
 {
     FILE *fp;
-    char *s, *p, sm[BUF_SIZE], *lhs, *rhs, msg[MSG_SIZE+50];
+
+    if (!filename) return 0;
+#ifdef RENVIRON_WIN32_STANDALONE
+    fp = fopen(filename, "rt");
+#else
+    fp = R_fopen(filename, "r");
+#endif
+    if (!fp) return 0;
+
+    char sm[BUF_SIZE], msg[MSG_SIZE];
+    const char *line_prefix = "\n      ";
+    const char *ignored_msg = "\n   They were ignored\n";
+    const char *truncated_msg = "[... truncated]";
+    const char *too_long = " (too long)";
     int errs = 0;
 
-    if (!filename || !(fp = R_fopen(filename, "r"))) return 0;
-    snprintf(msg, MSG_SIZE+50,
-	     "\n   File %s contains invalid line(s)", filename);
-
     while(fgets(sm, BUF_SIZE, fp)) {
-	sm[BUF_SIZE-1] = '\0';
-	s = rmspace(sm);
+	sm[BUF_SIZE-1] = '\0'; /* should not be needed */
+	/* embedded nulls are not supported */
+	int complete_line = feof(fp) || Renviron_strchr(sm, '\n');
+	char *s = rmspace(sm), *p;
 	if(strlen(s) == 0 || s[0] == '#') continue;
-	if(!(p = Rf_strchr(s, '='))) {
-	    errs++;
-	    if(strlen(msg) < MSG_SIZE) {
-		strcat(msg, "\n      "); strcat(msg, s);
+	if(!(p = Renviron_strchr(s, '=')) || !complete_line) {
+	    if(!errs) {
+		errs = 1;
+		Renviron_snprintf(msg, MSG_SIZE,
+			       "\n   File %s contains invalid line(s)",
+		               filename);
+	    }
+	    if (strlen(msg) + strlen(line_prefix) + strlen(s) +
+	        strlen(ignored_msg) < MSG_SIZE) {
+
+		strcat(msg, line_prefix);
+		strcat(msg, s);
+	    } else if (strlen(msg) + strlen(line_prefix) +
+		                45 + strlen(truncated_msg) +
+                                     strlen(ignored_msg) < MSG_SIZE) {
+		strcat(msg, line_prefix);
+		strncat(msg, s, 45);
+#ifndef RENVIRON_WIN32_STANDALONE
+		mbcsTruncateToValid(msg);
+#endif
+		strcat(msg, truncated_msg);
+	    }
+	    if (!complete_line) {
+		if (strlen(msg) + strlen(too_long) +
+		    strlen(ignored_msg) < MSG_SIZE) {
+
+		    strcat(msg, too_long);
+		}
+		/* skip the rest of the line */
+		while(!complete_line && fgets(sm, BUF_SIZE, fp)) {
+		    sm[BUF_SIZE-1] = '\0'; /* should not be needed */
+		    complete_line = feof(fp) || Renviron_strchr(sm, '\n');
+		}
+		if (!complete_line)
+		    break; /* error or EOF at line start */
 	    }
 	    continue;
 	}
 	*p = '\0';
-	lhs = rmspace(s);
-	rhs = findterm(rmspace(p+1));
+	char* lhs = rmspace(s),
+	    * rhs = findterm(rmspace(p+1));
 	/* set lhs = rhs */
 	if(strlen(lhs) && strlen(rhs)) Putenv(lhs, rhs);
     }
     fclose(fp);
     if (errs) {
-	strcat(msg, "\n   They were ignored\n");
-	R_ShowMessage(msg);
+	if (strlen(msg) + strlen(ignored_msg) < MSG_SIZE)
+	   strcat(msg, ignored_msg);
+	Renviron_warning(msg);
     }
     return 1;
 }
 
+#ifndef RENVIRON_WIN32_STANDALONE
 
 /* try system Renviron: R_HOME/etc/Renviron.  Unix only. */
-void process_system_Renviron()
+void process_system_Renviron(void)
 {
     char buf[PATH_MAX];
 
 #ifdef R_ARCH
     if(strlen(R_Home) + strlen("/etc/Renviron") + strlen(R_ARCH) + 1 > PATH_MAX - 1) {
-	R_ShowMessage("path to system Renviron is too long: skipping");
+	Renviron_warning("path to system Renviron is too long: skipping");
 	return;
     }
     strcpy(buf, R_Home);
@@ -231,14 +357,14 @@ void process_system_Renviron()
     strcat(buf, "/Renviron");
 #else
     if(strlen(R_Home) + strlen("/etc/Renviron") > PATH_MAX - 1) {
-	R_ShowMessage("path to system Renviron is too long: skipping");
+	Renviron_warning("path to system Renviron is too long: skipping");
 	return;
     }
     strcpy(buf, R_Home);
     strcat(buf, "/etc/Renviron");
 #endif
     if(!process_Renviron(buf))
-	R_ShowMessage("cannot find system Renviron");
+	Renviron_warning("cannot find system Renviron");
 }
 
 #ifdef HAVE_UNISTD_H
@@ -246,7 +372,7 @@ void process_system_Renviron()
 #endif
 
 /* try site Renviron: R_ENVIRON, then R_HOME/etc/Renviron.site. */
-void process_site_Renviron ()
+void process_site_Renviron (void)
 {
     char buf[PATH_MAX], *p = getenv("R_ENVIRON");
 
@@ -256,7 +382,7 @@ void process_site_Renviron ()
     }
 #ifdef R_ARCH
     if(strlen(R_Home) + strlen("/etc/Renviron.site") + strlen(R_ARCH) > PATH_MAX - 2) {
-	R_ShowMessage("path to arch-specific Renviron.site is too long: skipping");
+	Renviron_warning("path to arch-specific Renviron.site is too long: skipping");
     } else {
 	snprintf(buf, PATH_MAX, "%s/etc/%s/Renviron.site", R_Home, R_ARCH);
 	if(access(buf, R_OK) == 0) {
@@ -266,15 +392,19 @@ void process_site_Renviron ()
     }
 #endif
     if(strlen(R_Home) + strlen("/etc/Renviron.site") > PATH_MAX - 1) {
-	R_ShowMessage("path to Renviron.site is too long: skipping");
+	Renviron_warning("path to Renviron.site is too long: skipping");
 	return;
     }
     snprintf(buf, PATH_MAX, "%s/etc/Renviron.site", R_Home);
     process_Renviron(buf);
 }
 
+#ifdef Win32
+extern char *getRUser(void);
+#endif
+
 /* try user Renviron: ./.Renviron, then ~/.Renviron */
-void process_user_Renviron()
+void process_user_Renviron(void)
 {
     const char *s = getenv("R_ENVIRON_USER");
 
@@ -286,7 +416,7 @@ void process_user_Renviron()
 #ifdef R_ARCH
     char buff[100];
     snprintf(buff, 100, ".Renviron.%s", R_ARCH);
-    if( process_Renviron(buff)) return;
+    if(process_Renviron(buff)) return;
 #endif
     if(process_Renviron(".Renviron")) return;
 #ifdef Unix
@@ -296,10 +426,7 @@ void process_user_Renviron()
     {
 	char buf[1024]; /* MAX_PATH is less than this */
 	/* R_USER is not necessarily set yet, so we have to work harder */
-	s = getenv("R_USER");
-	if(!s) s = getenv("HOME");
-	if(!s) return;
-	snprintf(buf, 1024, "%s/.Renviron", s);
+	snprintf(buf, 1024, "%s/.Renviron", getRUser());
 	s = buf;
     }
 #endif
@@ -323,3 +450,5 @@ SEXP attribute_hidden do_readEnviron(SEXP call, SEXP op, SEXP args, SEXP env)
 	warning(_("file '%s' cannot be opened for reading"), fn);
     return ScalarLogical(res != 0);
 }
+
+#endif /* ^^^ not RENVIRON_WIN32_STANDALONE */
