@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000-2020  The R Core Team.
+ *  Copyright (C) 2000-2022  The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -78,8 +78,7 @@ those limits where there is a 64-bit time_t and the conversions work
 there is code below to extrapolate from 1902-2037.
 
 Path 2) was added for R 3.1.0 and is the only one supported on
-Windows: it is the default on macOS.  The only currently (Jan 2014)
-known OS with 64-bit time_t and complete tables is Linux.
+Windows: it is the default on macOS.
 
 */
 
@@ -109,6 +108,7 @@ extern char *tzname[2];
 
 /* Substitute based on glibc code. */
 #include "Rstrptime.h"
+/* --> Def.  R_strptime()  etc */
 
 static const int days_in_month[12] =
 {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
@@ -591,7 +591,7 @@ static void reset_tz(char *tz)
     tzset();
 }
 
-static void glibc_fix(stm *tm, int *invalid)
+static void glibc_fix(stm *tm, Rboolean *invalid)
 {
     /* set mon and mday which glibc does not always set.
        Use current year/... if none has been specified.
@@ -626,7 +626,7 @@ static void glibc_fix(stm *tm, int *invalid)
     } else {
 	if(tm->tm_mday == NA_INTEGER) {
 	    if(tm->tm_mon != NA_INTEGER) {
-		*invalid = 1;
+		*invalid = TRUE;
 		return;
 	    } else tm->tm_mday = tm0->tm_mday;
 	}
@@ -641,7 +641,7 @@ static const char ltnames [][7] =
 
 
 static void
-makelt(stm *tm, SEXP ans, R_xlen_t i, int valid, double frac_secs)
+makelt(stm *tm, SEXP ans, R_xlen_t i, Rboolean valid, double frac_secs)
 {
     if(valid) {
 	REAL(VECTOR_ELT(ans, 0))[i] = tm->tm_sec + frac_secs;
@@ -654,7 +654,7 @@ makelt(stm *tm, SEXP ans, R_xlen_t i, int valid, double frac_secs)
 	INTEGER(VECTOR_ELT(ans, 7))[i] = tm->tm_yday;
 	INTEGER(VECTOR_ELT(ans, 8))[i] = tm->tm_isdst;
     } else {
-	REAL(VECTOR_ELT(ans, 0))[i] = NA_REAL;
+	REAL(VECTOR_ELT(ans, 0))[i] = frac_secs;
 	for(int j = 1; j < 8; j++)
 	    INTEGER(VECTOR_ELT(ans, j))[i] = NA_INTEGER;
 	INTEGER(VECTOR_ELT(ans, 8))[i] = -1;
@@ -670,15 +670,14 @@ makelt(stm *tm, SEXP ans, R_xlen_t i, int valid, double frac_secs)
 SEXP attribute_hidden do_asPOSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP stz, x, ans, ansnames, klass, tzone;
-    int isgmt = 0, valid, settz = 0;
+    int isgmt = 0, settz = 0;
     char oldtz[1001] = "";
-    const char *tz = NULL;
 
     checkArity(op, args);
     PROTECT(x = coerceVector(CAR(args), REALSXP));
     if(!isString((stz = CADR(args))) || LENGTH(stz) != 1)
 	error(_("invalid '%s' value"), "tz");
-    tz = CHAR(STRING_ELT(stz, 0));
+    const char *tz = CHAR(STRING_ELT(stz, 0));
     if(strlen(tz) == 0) {
 	/* do a direct look up here as this does not otherwise
 	   work on Windows */
@@ -730,17 +729,20 @@ SEXP attribute_hidden do_asPOSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
     for(R_xlen_t i = 0; i < n; i++) {
 	stm dummy, *ptm = &dummy;
 	double d = REAL(x)[i];
+	Rboolean valid;
 	if(R_FINITE(d)) {
 	    ptm = localtime0(&d, !isgmt, &dummy);
 	    /* in theory localtime/gmtime always return a valid
 	       struct tm pointer, but Windows uses NULL for error
 	       conditions (like negative times). */
 	    valid = (ptm != NULL);
-	} else valid = 0;
-	makelt(ptm, ans, i, valid, d - floor(d));
+	} else {
+	    valid = FALSE;
+	}
+	makelt(ptm, ans, i, valid, valid ? d - floor(d) : d);
 	if(!isgmt) {
 	    char *p = "";
-	    // or ptm->tm_zone
+	    // or ptm->tm_zone (but not specified by POSIX)
 	    if(valid && ptm->tm_isdst >= 0)
 		p = R_tzname[ptm->tm_isdst];
 	    SET_STRING_ELT(VECTOR_ELT(ans, 9), i, mkChar(p));
@@ -763,25 +765,23 @@ SEXP attribute_hidden do_asPOSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
     return ans;
 }
 
+#define check_nlen(_i_)							\
+  if(nlen[_i_] == 0)							\
+    error(_("zero-length component [[%d]] in non-empty \"POSIXlt\" structure"), (_i_)+1)
+
 // .Internal(as.POSIXct(x, tz)) -- called only from  as.POSIXct.POSIXlt()
 SEXP attribute_hidden do_asPOSIXct(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP stz, x, ans;
-    R_xlen_t n = 0, nlen[9];
-    int isgmt = 0, settz = 0;
-    char oldtz[1001] = "";
-    const char *tz = NULL;
-    stm tm;
-    double tmp;
-
     checkArity(op, args);
-    PROTECT(x = duplicate(CAR(args))); /* coerced below */
+
+    SEXP x = PROTECT(duplicate(CAR(args))); /* coerced below */
     if(!isVectorList(x) || LENGTH(x) < 9) // must be 'POSIXlt'
 	error(_("invalid '%s' argument"), "x");
+    SEXP stz;
     if(!isString((stz = CADR(args))) || LENGTH(stz) != 1)
 	error(_("invalid '%s' value"), "tz");
 
-    tz = CHAR(STRING_ELT(stz, 0));
+    const char *tz = CHAR(STRING_ELT(stz, 0));
     if(strlen(tz) == 0) {
 	/* do a direct look up here as this does not otherwise
 	   work on Windows */
@@ -791,8 +791,11 @@ SEXP attribute_hidden do_asPOSIXct(SEXP call, SEXP op, SEXP args, SEXP env)
 	    tz = CHAR(STRING_ELT(stz, 0));
 	}
     }
+
     PROTECT(stz); /* it might be new */
-    if(strcmp(tz, "GMT") == 0  || strcmp(tz, "UTC") == 0) isgmt = 1;
+    int isgmt = (strcmp(tz, "GMT") == 0  || strcmp(tz, "UTC") == 0) ? 1 : 0;
+    char oldtz[1001] = "";
+    int settz = 0;
     if(!isgmt && strlen(tz) > 0) settz = set_tz(tz, oldtz);
 #ifdef USE_INTERNAL_MKTIME
     else R_tzsetwall(); // to get the system timezone recorded
@@ -800,27 +803,25 @@ SEXP attribute_hidden do_asPOSIXct(SEXP call, SEXP op, SEXP args, SEXP env)
     tzset();
 #endif
 
+    R_xlen_t n = 0, nlen[9];
     for(int i = 0; i < 6; i++)
 	if((nlen[i] = XLENGTH(VECTOR_ELT(x, i))) > n) n = nlen[i];
     if((nlen[8] = XLENGTH(VECTOR_ELT(x, 8))) > n) n = nlen[8];
     if(n > 0) {
 	for(int i = 0; i < 6; i++)
-	    if(nlen[i] == 0)
-		error(_("zero-length component [[%d]] in non-empty \"POSIXlt\" structure"),
-		      i+1);
-	if(nlen[8] == 0)
-	    error(_("zero-length component [[%d]] in non-empty \"POSIXlt\" structure"), 9);
+	    check_nlen(i);
+	check_nlen(8);
     }
     /* coerce fields to integer or real */
     SET_VECTOR_ELT(x, 0, coerceVector(VECTOR_ELT(x, 0), REALSXP));
-    for(int i = 0; i < 6; i++)
-	SET_VECTOR_ELT(x, i, coerceVector(VECTOR_ELT(x, i),
-					  i > 0 ? INTSXP: REALSXP));
+    for(int i = 1; i < 6; i++)
+	SET_VECTOR_ELT(x, i, coerceVector(VECTOR_ELT(x, i), INTSXP));
     SET_VECTOR_ELT(x, 8, coerceVector(VECTOR_ELT(x, 8), INTSXP));
 
-    PROTECT(ans = allocVector(REALSXP, n));
+    SEXP ans = PROTECT(allocVector(REALSXP, n));
     for(R_xlen_t i = 0; i < n; i++) {
 	double secs = REAL(VECTOR_ELT(x, 0))[i%nlen[0]], fsecs = floor(secs);
+	stm tm;
 	// avoid (int) NAN
 	tm.tm_sec   = R_FINITE(secs) ? (int) fsecs: NA_INTEGER;
 	tm.tm_min   = INTEGER(VECTOR_ELT(x, 1))[i%nlen[1]];
@@ -830,13 +831,15 @@ SEXP attribute_hidden do_asPOSIXct(SEXP call, SEXP op, SEXP args, SEXP env)
 	tm.tm_year  = INTEGER(VECTOR_ELT(x, 5))[i%nlen[5]];
 	/* mktime ignores tm.tm_wday and tm.tm_yday */
 	tm.tm_isdst = isgmt ? 0 : INTEGER(VECTOR_ELT(x, 8))[i%nlen[8]];
-	if(!R_FINITE(secs) || tm.tm_min == NA_INTEGER ||
-	   tm.tm_hour == NA_INTEGER || tm.tm_mday == NA_INTEGER ||
-	   tm.tm_mon == NA_INTEGER || tm.tm_year == NA_INTEGER)
+	if(!R_FINITE(secs))
+	    REAL(ans)[i] = secs;
+	else if(tm.tm_min  == NA_INTEGER ||
+		tm.tm_hour == NA_INTEGER || tm.tm_mday == NA_INTEGER ||
+		tm.tm_mon  == NA_INTEGER || tm.tm_year == NA_INTEGER)
 	    REAL(ans)[i] = NA_REAL;
 	else {
 	    errno = 0;
-	    tmp = mktime0(&tm, !isgmt);
+	    double tmp = mktime0(&tm, !isgmt);
 #ifdef MKTIME_SETS_ERRNO
 	    REAL(ans)[i] = errno ? NA_REAL : tmp + (secs - fsecs);
 #else
@@ -875,6 +878,8 @@ SEXP attribute_hidden do_formatPOSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
     if(UseTZ == NA_LOGICAL)
 	error(_("invalid '%s' argument"), "usetz");
     SEXP tz = getAttrib(x, install("tzone"));
+    if(!isNull(tz) && !isString(tz))
+	error(_("invalid '%s'"), "attr(x, \"tzone\")");
 
     const char *tz1;
     if (!isNull(tz) && strlen(tz1 = CHAR(STRING_ELT(tz, 0)))) {
@@ -908,9 +913,7 @@ SEXP attribute_hidden do_formatPOSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
     }
     if(n > 0) {
 	for(int i = 0; i < nn; i++)
-	    if(nlen[i] == 0)
-		error(_("zero-length component [[%d]] in non-empty \"POSIXlt\" structure"),
-		      i+1);
+	  check_nlen(i);
     }
     R_xlen_t N = (n > 0) ? ((m > n) ? m : n) : 0;
     SEXP ans = PROTECT(allocVector(STRSXP, N));
@@ -923,6 +926,7 @@ SEXP attribute_hidden do_formatPOSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
     if(have_zone && !isString(VECTOR_ELT(x, 9)))
 	error(_("invalid component [[10]] in \"POSIXlt\" should be 'zone'"));
     if(!have_zone && LENGTH(x) > 9) // rather even error ?
+	/* never when !HAVE_GMTOFF */
 	warning(_("More than 9 list components in \"POSIXlt\" without timezone"));
     for(R_xlen_t i = 0; i < N; i++) {
 	double secs = REAL(VECTOR_ELT(x, 0))[i%nlen[0]], fsecs = floor(secs);
@@ -955,7 +959,12 @@ SEXP attribute_hidden do_formatPOSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
 	    tm.tm_gmtoff = (tmp == NA_INTEGER) ? 0 : tmp;
 #endif
 	}
-	if(!R_FINITE(secs) || tm.tm_min == NA_INTEGER ||
+	if(!R_FINITE(secs) && tm.tm_year == NA_INTEGER) {
+	    SET_STRING_ELT(ans, i,
+			   ISNA(secs)  ? NA_STRING :
+			   ISNAN(secs) ? mkChar("NaN") :
+			   (secs > 0)  ? mkChar("Inf") : mkChar("-Inf"));
+	} else if(!R_FINITE(secs) || tm.tm_min == NA_INTEGER ||
 	   tm.tm_hour == NA_INTEGER || tm.tm_mday == NA_INTEGER ||
 	   tm.tm_mon == NA_INTEGER || tm.tm_year == NA_INTEGER) {
 	    SET_STRING_ELT(ans, i, NA_STRING);
@@ -1043,10 +1052,9 @@ SEXP attribute_hidden do_formatPOSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
 // .Internal(strptime(as.character(x), format, tz))
 SEXP attribute_hidden do_strptime(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP x, sformat, ans, ansnames, klass, stz, tzone = R_NilValue;
-    int invalid, isgmt = 0, settz = 0, offset;
+    SEXP x, sformat, klass, stz, tzone = R_NilValue;
+    int isgmt = 0, settz = 0, offset;
     stm tm, tm2, *ptm = &tm;
-    const char *tz = NULL;
     char oldtz[1001] = "";
     double psecs = 0.0;
     R_xlen_t n, m, N;
@@ -1058,7 +1066,7 @@ SEXP attribute_hidden do_strptime(SEXP call, SEXP op, SEXP args, SEXP env)
 	error(_("invalid '%s' argument"), "format");
     if(!isString((stz = CADDR(args))) || LENGTH(stz) != 1)
 	error(_("invalid '%s' value"), "tz");
-    tz = CHAR(STRING_ELT(stz, 0));
+    const char *tz = CHAR(STRING_ELT(stz, 0));
     if(strlen(tz) == 0) {
 	/* do a direct look up here as this does not otherwise
 	   work on Windows */
@@ -1096,7 +1104,7 @@ SEXP attribute_hidden do_strptime(SEXP call, SEXP op, SEXP args, SEXP env)
 #else
     int nans = 10 - isgmt;
 #endif
-    PROTECT(ans = allocVector(VECSXP, nans));
+    SEXP ans = PROTECT(allocVector(VECSXP, nans));
     for(int i = 0; i < 9; i++)
 	SET_VECTOR_ELT(ans, i, allocVector(i > 0 ? INTSXP : REALSXP, N));
     if(!isgmt) {
@@ -1106,7 +1114,7 @@ SEXP attribute_hidden do_strptime(SEXP call, SEXP op, SEXP args, SEXP env)
 #endif
     }
 
-    PROTECT(ansnames = allocVector(STRSXP, nans));
+    SEXP ansnames = PROTECT(allocVector(STRSXP, nans));
     for(int i = 0; i < nans; i++)
 	SET_STRING_ELT(ansnames, i, mkChar(ltnames[i]));
 
@@ -1123,7 +1131,8 @@ SEXP attribute_hidden do_strptime(SEXP call, SEXP op, SEXP args, SEXP env)
 	tm.tm_isdst = -1;
 #endif
 	offset = NA_INTEGER;
-	invalid = STRING_ELT(x, i%n) == NA_STRING ||
+	Rboolean invalid =
+	    STRING_ELT(x, i%n) == NA_STRING ||
 	    !R_strptime(translateChar(STRING_ELT(x, i%n)),
 			translateChar(STRING_ELT(sformat, i%m)),
 			&tm, &psecs, &offset);
@@ -1147,7 +1156,7 @@ SEXP attribute_hidden do_strptime(SEXP call, SEXP op, SEXP args, SEXP env)
 		if (t0 != -1) {
 		    t0 -= offset; /* offset = -0800 is Seattle */
 		    ptm = localtime0(&t0, 1-isgmt, &tm2);
-		} else invalid = 1;
+		} else invalid = TRUE;
 	    } else {
 		/* we do want to set wday, yday, isdst, but not to
 		   adjust structure at DST boundaries */
@@ -1159,7 +1168,7 @@ SEXP attribute_hidden do_strptime(SEXP call, SEXP op, SEXP args, SEXP env)
 	    }
 	    invalid = validate_tm(&tm) != 0;
 	}
-	makelt(ptm, ans, i, !invalid, psecs - floor(psecs));
+	makelt(ptm, ans, i, !invalid, invalid ? NA_REAL : psecs - floor(psecs));
 	if(!isgmt) {
 	    const char *p = "";
 	    if(!invalid && tm.tm_isdst >= 0) {
@@ -1192,13 +1201,11 @@ SEXP attribute_hidden do_strptime(SEXP call, SEXP op, SEXP args, SEXP env)
 SEXP attribute_hidden do_D2POSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP x, ans, ansnames, klass;
-    R_xlen_t n;
-    int valid, day, y, tmp, mon;
     stm tm;
 
     checkArity(op, args);
     PROTECT(x = coerceVector(CAR(args), REALSXP));
-    n = XLENGTH(x);
+    R_xlen_t n = XLENGTH(x);
     PROTECT(ans = allocVector(VECSXP, 9));
     for(int i = 0; i < 9; i++)
 	SET_VECTOR_ELT(ans, i, allocVector(i > 0 ? INTSXP : REALSXP, n));
@@ -1208,14 +1215,16 @@ SEXP attribute_hidden do_D2POSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
 	SET_STRING_ELT(ansnames, i, mkChar(ltnames[i]));
 
     for(R_xlen_t i = 0; i < n; i++) {
-	if(R_FINITE(REAL(x)[i])) {
-	    day = (int) floor(REAL(x)[i]);
+	double x_i = REAL(x)[i];
+	Rboolean valid = R_FINITE(x_i);
+	if(valid) {
+	    int day = (int) floor(x_i);
 	    tm.tm_hour = tm.tm_min = tm.tm_sec = 0;
 	    /* weekday: 1970-01-01 was a Thursday */
 	    if ((tm.tm_wday = ((4 + day) % 7)) < 0) tm.tm_wday += 7;
 
 	    /* year & day within year */
-	    y = 1970;
+	    int y = 1970, tmp, mon;
 	    if (day >= 0)
 		for ( ; day >= (tmp = days_in_year(y)); day -= tmp, y++);
 	    else
@@ -1232,10 +1241,8 @@ SEXP attribute_hidden do_D2POSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
 	    tm.tm_mon = mon;
 	    tm.tm_mday = day + 1;
 	    tm.tm_isdst = 0; /* no dst in GMT */
-
-	    valid = 1;
-	} else valid = 0;
-	makelt(&tm, ans, i, valid, 0.0);
+	}
+	makelt(&tm, ans, i, valid, valid ? 0.0 : x_i);
     }
     setAttrib(ans, R_NamesSymbol, ansnames);
     PROTECT(klass = allocVector(STRSXP, 2));
@@ -1253,32 +1260,29 @@ SEXP attribute_hidden do_D2POSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
 
 SEXP attribute_hidden do_POSIXlt2D(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP x, ans, klass;
-    R_xlen_t n = 0, nlen[9];
-    stm tm;
-
     checkArity(op, args);
-    PROTECT(x = duplicate(CAR(args)));
+    SEXP x = PROTECT(duplicate(CAR(args)));
     if(!isVectorList(x) || LENGTH(x) < 9)
 	error(_("invalid '%s' argument"), "x");
 
+    R_xlen_t n = 0, nlen[9];
     for(int i = 3; i < 6; i++)
 	if((nlen[i] = XLENGTH(VECTOR_ELT(x, i))) > n) n = nlen[i];
-    if((nlen[8] = XLENGTH(VECTOR_ELT(x, 8))) > n) n = nlen[8];
+    if((nlen[0] = XLENGTH(VECTOR_ELT(x, 0))) > n) n = nlen[0]; // sec; need for Inf,NaN,..
+    if((nlen[8] = XLENGTH(VECTOR_ELT(x, 8))) > n) n = nlen[8]; // isdst
     if(n > 0) {
+	check_nlen(0);
 	for(int i = 3; i < 6; i++)
-	    if(nlen[i] == 0)
-		error(_("zero-length component [[%d]] in non-empty \"POSIXlt\" structure"),
-		      i+1);
-	if(nlen[8] == 0)
-	    error(_("zero-length component [[%d]] in non-empty \"POSIXlt\" structure"), 9);
+	    check_nlen(i);
+	check_nlen(8);
     }
-    /* coerce relevant fields to integer */
+    /* coerce relevant fields [3,4,5] = [mday,mon,year] to integer */
     for(int i = 3; i < 6; i++)
 	SET_VECTOR_ELT(x, i, coerceVector(VECTOR_ELT(x, i), INTSXP));
 
-    PROTECT(ans = allocVector(REALSXP, n));
+    SEXP ans = PROTECT(allocVector(REALSXP, n));
     for(R_xlen_t i = 0; i < n; i++) {
+	stm tm;
 	tm.tm_sec = tm.tm_min = tm.tm_hour = 0;
 	tm.tm_mday  = INTEGER(VECTOR_ELT(x, 3))[i%nlen[3]];
 	tm.tm_mon   = INTEGER(VECTOR_ELT(x, 4))[i%nlen[4]];
@@ -1286,8 +1290,8 @@ SEXP attribute_hidden do_POSIXlt2D(SEXP call, SEXP op, SEXP args, SEXP env)
 	/* mktime ignores tm.tm_wday and tm.tm_yday */
 	tm.tm_isdst = 0;
 	if(tm.tm_mday == NA_INTEGER || tm.tm_mon == NA_INTEGER ||
-	   tm.tm_year == NA_INTEGER || validate_tm(&tm) < 0)
-	    REAL(ans)[i] = NA_REAL;
+	   tm.tm_year == NA_INTEGER || validate_tm(&tm) < 0) // non-finite: take the 'sec'
+	    REAL(ans)[i] = REAL(VECTOR_ELT(x, 0))[i%nlen[0]];
 	else {
 	    /* -1 must be error as seconds were zeroed */
 	    double tmp = mktime00(&tm);
@@ -1297,7 +1301,7 @@ SEXP attribute_hidden do_POSIXlt2D(SEXP call, SEXP op, SEXP args, SEXP env)
 
     SEXP nm = getAttrib(VECTOR_ELT(x, 5), R_NamesSymbol);
     if (nm != R_NilValue) setAttrib(ans, R_NamesSymbol, nm);
-    PROTECT(klass = mkString("Date"));
+    SEXP klass = PROTECT(mkString("Date"));
     classgets(ans, klass);
     UNPROTECT(3);
     return ans;
