@@ -4,15 +4,17 @@
 if(.Platform$OS.type == "unix" &&
    is.null(nsl("cran.r-project.org"))) q()
 
-# test do_download.
-nrow(available.packages(contrib.url("http://cran.r-project.org")))
+# test do_download (and "record" #{packages}):
+ap <- available.packages(contrib.url("http://cran.r-project.org"))
+## IGNORE_RDIFF_BEGIN
+nrow(ap)
+## IGNORE_RDIFF_END
 
 # test url connections on http
 zz <- url("http://cran.r-project.org/")
 readLines(zz)
 close(zz)
 
-# and via read.table, test http and ftp.
+# and via read.table
 
 read.table("http://www.stats.ox.ac.uk/pub/datasets/csb/ch11b.dat")
-read.table("ftp://ftp.stats.ox.ac.uk/pub/datasets/csb/ch11b.dat")
